import typing

import FreeCAD
import Spreadsheet

SequenceNone_t: typing.TypeAlias = tuple[None, typing.Any]
SequenceDoc_t: typing.TypeAlias = tuple[FreeCAD.DocumentObject, str | typing.Sequence[str]]
PropX_t: typing.TypeAlias = None | FreeCAD.DocumentObject | SequenceNone_t | SequenceDoc_t


# AssemblyLinkPy.xml
class AssemblyLink(FreeCAD.Part):
    """This class handles document objects in Assembly"""

    @property
    def LinkedObject(self) -> None | FreeCAD.DocumentObject | tuple[FreeCAD.DocumentObject, str | list[str]]:
        """
        Property group: General.
        Property TypeId: App::PropertyXLink.
        The linked assembly.
        """

    @LinkedObject.setter
    def LinkedObject(self, value: PropX_t): ...

    @property
    def Rigid(self) -> bool:
        """
        Property group: General.
        Property TypeId: App::PropertyBool.
        If the sub-assembly is set to Rigid, it will act as a rigid body. Else its joints will be taken into account.
        """

    @Rigid.setter
    def Rigid(self, value: int | bool): ...


# BomObjectPy.xml
class BomObject(Spreadsheet.Sheet):
    """This class is the BOM object of assemblies, it derives from Spreadsheet::Sheet."""

    @property
    def columnsNames(self) -> list[str]:
        """
        Property group: Bom.
        Property TypeId: App::PropertyStringList.
        List of the columns of the Bill of Materials.
        """

    @columnsNames.setter
    def columnsNames(self, value: typing.Iterable[str] | dict[int, str]): ...

    @property
    def detailParts(self) -> bool:
        """
        Property group: Bom.
        Property TypeId: App::PropertyBool.
        Detail Parts sub-components.
        """

    @detailParts.setter
    def detailParts(self, value: int | bool): ...

    @property
    def detailSubAssemblies(self) -> bool:
        """
        Property group: Bom.
        Property TypeId: App::PropertyBool.
        Detail sub-assemblies components.
        """

    @detailSubAssemblies.setter
    def detailSubAssemblies(self, value: int | bool): ...

    @property
    def onlyParts(self) -> bool:
        """
        Property group: Bom.
        Property TypeId: App::PropertyBool.
        Only Part containers will be added. Solids like PartDesign Bodies will be ignored.
        """

    @onlyParts.setter
    def onlyParts(self, value: int | bool): ...


# BomGroupPy.xml
class BomGroup(FreeCAD.DocumentObjectGroup):
    """This class is a group subclass for boms."""


# AssemblyObjectPy.xml
class AssemblyObject(FreeCAD.Part):
    """This class handles document objects in Assembly"""

    def clearUndo(self):
        """
        Clear the registered undo positions.

                  clearUndo()

                  Returns: None
        """

    def ensureIdentityPlacements(self):
        """
        Makes sure that LinkGroups or sub-assemblies have identity placements.

                  ensureIdentityPlacements()

                  Returns: None
        """

    def exportAsASMT(self, utf8Name: str, /):
        """
        Export the assembly in a text format called ASMT.

                  exportAsASMT(fileName:str)

                  Args:
                  fileName: The name of the file where the ASMT will be exported.
        
        Possible exceptions: (ValueError).
        """

    def isJointConnectingPartToGround(self, pyobj, pname: str, /) -> bool:
        """
        Check if a joint is connecting a part to the ground.

                  isJointConnectingPartToGround(joint, propName) -> bool

                  Args:
                  - joint: document object of the joint to check.
                  - propName: string 'Part1' or 'Part2' of the joint.

                  Returns: True if part is connected to ground
        """

    def isPartConnected(self, pyobj, /) -> bool:
        """
        Check if a part is connected to the ground through joints.

                  isPartConnected(obj) -> bool

                  Args: document object to check.

                  Returns: True if part is connected to ground
        """

    def isPartGrounded(self, pyobj, /) -> bool:
        """
        Check if a part has a grounded joint.

                  isPartGrounded(obj) -> bool

                  Args:
                  - obj: document object of the part to check.

                  Returns: True if part has grounded joint
        """

    @typing.overload
    def solve(self, enableUndoPy: bool, /) -> int: ...

    @typing.overload
    def solve(self) -> int:
        """
        Solve the assembly and update part placements.

                  solve(enableRedo=False) -> int

                  Args:
                  enableRedo: Whether the solve save the initial position of parts
                  to enable undoing it even without a transaction.
                  Defaults to `False` ie the solve cannot be undone if called
                  outside of a transaction.

                  Returns:
                  0 in case of success, otherwise the following codes in this order of
                  priority:
                  -6 if no parts are fixed.
                  -4 if over-constrained,
                  -3 if conflicting constraints,
                  -5 if malformed constraints
                  -1 if solver error,
                  -2 if redundant constraints.
        """

    def undoSolve(self):
        """
        Undo the last solve of the assembly and return part placements to their initial position.

                  undoSolve()

                  Returns: None
        """


# ViewGroupPy.xml
class ViewGroup(FreeCAD.DocumentObjectGroup):
    """This class is a group subclass for joints."""


# JointGroupPy.xml
class JointGroup(FreeCAD.DocumentObjectGroup):
    """This class is a group subclass for joints."""
