import typing

import FreeCAD
import FreeCADGui

_T = typing.TypeVar("_T")
Triple_t: typing.TypeAlias = tuple[_T, _T, _T]
Quadruple_t: typing.TypeAlias = tuple[_T, _T, _T, _T]


# ViewProviderPartExtPy.xml
class ViewProviderPartExt(FreeCADGui.ViewProviderGeometryObject):
    """This is the ViewProvider geometry class"""

    @property
    def AngularDeflection(self) -> FreeCAD.Quantity:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyAngle.

        Specify how finely to generate the mesh for rendering on screen or when exporting.
        The default value is 28.5 degrees, or 0.5 radians. The smaller the value
        the smoother the appearance in the 3D view, and the finer the mesh that will be exported.
        """

    @AngularDeflection.setter
    def AngularDeflection(self, value: str | float | FreeCAD.Quantity): ...

    @property
    def Deviation(self) -> float:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyFloatConstraint.

        Sets the accuracy of the polygonal representation of the model
        in the 3D view (tessellation). Lower values indicate better quality.
        The value is in percent of object's size.
        """

    @Deviation.setter
    def Deviation(self, value: float | Quadruple_t[float]): ...

    @property
    def DrawStyle(self) -> int:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyEnumeration.
        Defines the style of the edges in the 3D view.
        """

    @DrawStyle.setter
    def DrawStyle(self, value: typing.Literal['Solid', 'Dashed', 'Dotted', 'Dashdot']): ...

    @property
    def Lighting(self) -> int:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyEnumeration.
        Set object lighting.
        """

    @Lighting.setter
    def Lighting(self, value: typing.Literal['One side', 'Two side']): ...

    @property
    def LineColor(self) -> tuple[float, float, float, float]:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyColor.
        Set object line color.
        """

    @LineColor.setter
    def LineColor(self, value: Triple_t[float] | Quadruple_t[float] | int): ...

    @property
    def LineColorArray(self) -> list[tuple[float, float, float, float]]:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyColorList.
        Object line color array.
        """

    @LineColorArray.setter
    def LineColorArray(self, value: typing.Iterable[Triple_t[float] | Quadruple_t[float] | int] | dict[int, Triple_t[float] | Quadruple_t[float] | int]): ...

    @property
    def LineMaterial(self) -> FreeCAD.Material:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyMaterial.
        Object line material.
        """

    @LineMaterial.setter
    def LineMaterial(self, value: FreeCAD.Material): ...

    @property
    def LineWidth(self) -> float:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyFloatConstraint.
        Set object line width.
        """

    @LineWidth.setter
    def LineWidth(self, value: float | Quadruple_t[float]): ...

    @property
    def PointColor(self) -> tuple[float, float, float, float]:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyColor.
        Set object point color.
        """

    @PointColor.setter
    def PointColor(self, value: Triple_t[float] | Quadruple_t[float] | int): ...

    @property
    def PointColorArray(self) -> list[tuple[float, float, float, float]]:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyColorList.
        Object point color array.
        """

    @PointColorArray.setter
    def PointColorArray(self, value: typing.Iterable[Triple_t[float] | Quadruple_t[float] | int] | dict[int, Triple_t[float] | Quadruple_t[float] | int]): ...

    @property
    def PointMaterial(self) -> FreeCAD.Material:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyMaterial.
        Object point material.
        """

    @PointMaterial.setter
    def PointMaterial(self, value: FreeCAD.Material): ...

    @property
    def PointSize(self) -> float:
        """
        Property group: Object Style.
        Property TypeId: App::PropertyFloatConstraint.
        Set object point size.
        """

    @PointSize.setter
    def PointSize(self, value: float | Quadruple_t[float]): ...
